
/*
  Cubic spline.
  For course 'Advanced scientifc computing 2005'
          A.Kuronen

*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>


double f(double x);
double df(double x);
void Spline3_Coeff(int n,double *x,double *y,double *z);
double Spline3_Eval(int n, double *t, double *y, double *z, double x0);


main (int argc, char **argv)
{
  double x,x1,x2,dx,y,fpo,fpn,*xa,*ya,*za;
  int n,np,nat,i,j;
  FILE *fk,*fi,*ff,*fd;

  if (argc!=7) {
    fprintf(stderr,"Usage: %s x1 x2 dx fpo fpn nat\n",argv[0]); return (1);
  }
  x1=atof(*++argv);
  x2=atof(*++argv);
  dx=atof(*++argv);
  fpo=atof(*++argv);
  fpn=atof(*++argv);
  nat=atoi(*++argv);

  if (nat!=0) {
    fprintf(stderr,"Clamped splines not yet implemented.\n");
    exit(-1);
  }

  fk=fopen("cubicspline_dat.knots","w");
  fi=fopen("cubicspline_dat.inter","w");
  ff=fopen("cubicspline_dat.func","w");
  fd=fopen("cubicspline_dat.diff","w");

  n=(x2-x1)/dx+2;
  xa = (double *) malloc((size_t) (n*sizeof(double)));  
  ya = (double *) malloc((size_t) (n*sizeof(double)));  
  za = (double *) malloc((size_t) (n*sizeof(double)));  
  fprintf(stderr,"n %d\n",n);
  for (i=0;i<n;i++) {
    x=x1+dx*i;
    xa[i]=x;
    ya[i]=f(x);
    fprintf(fk,"%g %g\n",xa[i],ya[i]);
  }

  Spline3_Coeff(n,xa,ya,za);
  //for (i=0;i<n;i++) fprintf(stdout,"xxx %g %g %g\n",xa[i],ya[i],za[i]);

  dx/=20.;
  np=(x2-x1)/dx+2;
  for (i=0;i<np;i++) {
    x=x1+dx*i;
    y=Spline3_Eval(n,xa,ya,za,x);
    fprintf(fi,"%g %g\n",x,y);
    fprintf(ff,"%g %g\n",x,f(x));
    fprintf(fd,"%g %g\n",x,y-f(x));
  } 

  fclose(fk);
  fclose(fi);
  fclose(ff);
  fclose(fd);

}


void Spline3_Coeff(int n,double *x,double *y,double *z)
{
  int i;
  double *h, *b, *u, *v;
  h = (double *) malloc((size_t) (n*sizeof(double)));
  b = (double *) malloc((size_t) (n*sizeof(double)));
  u = (double *) malloc((size_t) (n*sizeof(double)));
  v = (double *) malloc((size_t) (n*sizeof(double)));

  for (i=0; i<n; i++){
    h[i] = x[i+1]-x[i];
    b[i] = (y[i+1]-y[i])/h[i];
  }
  u[1]= 2.0*(h[1]+h[0]);
  v[1]= 6.0*(b[1]-b[0]);
  for (i=2; i<n; i++){
    u[i]= 2.0*(h[i]+h[i-1])-h[i-1]*h[i-1]/u[i-1];
    v[i]= 6.0*(b[i]-b[i-1])-h[i-1]*v[i-1]/u[i-1];
  }
  z[n]=0;
  for (i=n-1; i>0; i--){
    z[i] = (v[i]-h[i]*z[i+1])/u[i];
  }
  z[0]=0;
  //for (i=0;i<n;i++) fprintf(stderr,"yyy %g %g %g %g\n",h[i],b[i],u[i],v[i]);
  //free(h); free(b); free(u); free(v);
}


double Spline3_Eval(int n, double *x, double *y, double *z, double x0)
{
  int i;
  double h, tmp, result;
  for (i=n-1; i>=0; i--) {
    if (x0-x[i]>=0) break;
  }
  h = x[i+1]-x[i];
  tmp = 0.5*z[i] + (x0-x[i])*(z[i+1]-z[i])/(6.0*h);
  tmp = -(h/6.0)*(z[i+1]+2.0*z[i])+(y[i+1]-y[i])/h + (x0-x[i])*tmp;
  result = y[i] + (x0-x[i])*tmp;
  return(result);
}


double f(double x) {
  return sin(x);
}
